==> mule-datasense-4.3.0-20220919-sources.jar: org/mule/datasense/impl/DataSenseProviderResolver.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.datasense.impl;

import org.mule.datasense.api.metadataprovider.ApplicationModel;
import org.mule.datasense.api.metadataprovider.DataSenseProvider;
import org.mule.datasense.catalog.builder.TypesCatalogBuilderContext;
import org.mule.datasense.catalog.loader.xml.TypesCatalogXmlLoader;
import org.mule.datasense.catalog.model.TypesCatalog;
import org.mule.datasense.common.loader.xml.XmlUtils;
import org.mule.datasense.common.util.Notifier;
import org.mule.datasense.declarations.loader.TypeDeclarationLoaderContext;
import org.mule.datasense.declarations.loader.xml.ExtensionOperationTypeDeclarationXmlLoader;
import org.mule.datasense.declarations.loader.xml.MessageProcessorTypeDeclarationXmlLoader;
import org.mule.datasense.declarations.loader.xml.TypeDeclarationXmlLoader;
import org.mule.datasense.declarations.model.ExtensionOperationTypeDeclaration;
import org.mule.datasense.declarations.model.MessageProcessorTypeDeclaration;
import org.mule.datasense.enrichment.loader.ComponentModelEnrichmentLoaderContext;
import org.mule.datasense.enrichment.loader.xml.ComponentModelEnrichmentXmlLoader;
import org.mule.datasense.enrichment.model.ComponentModelEnrichments;
import org.mule.datasense.impl.model.ast.AstNodeLocation;
import org.mule.datasense.impl.model.ast.AstNotification;
import org.mule.datasense.impl.model.ast.MessageProcessorNode;
import org.mule.datasense.impl.model.reporting.NotificationMessages;
import org.mule.datasense.impl.phases.typing.resolver.GlobalBindingMetadataTypes;
import org.mule.runtime.api.component.ComponentIdentifier;
import org.mule.runtime.api.component.location.ComponentLocation;
import org.mule.runtime.api.i18n.I18nMessage;
import org.mule.runtime.api.meta.model.operation.OperationModel;
import org.mule.runtime.api.meta.model.source.SourceModel;
import org.mule.runtime.api.metadata.ExpressionLanguageMetadataService;
import org.mule.runtime.apikit.metadata.api.Metadata;
import org.mule.runtime.apikit.metadata.api.MetadataBuilder;
import org.mule.runtime.ast.api.ComponentAst;

import java.io.StringReader;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

import org.w3c.dom.Element;
import org.xml.sax.InputSource;

import com.google.common.base.Throwables;

/**
 *
 */
public class DataSenseProviderResolver {

  private static final List<ComponentIdentifier> componentIdentifierBlockList = Collections.emptyList();

  public static boolean isDynamicDataSenseSupportedFor(MessageProcessorNode messageProcessorNode) {
    return !componentIdentifierBlockList.contains(messageProcessorNode.getComponentIdentifier());
  }

  private final DataSenseProvider dataSenseProvider;
  private final ApplicationModelResolver applicationModelResolver;
  private final AstNotification astNotification;
  private final Notifier notifier;

  private final TypesCatalog typesCatalog;
  private final ComponentModelEnrichments componentModelEnrichments;
  private final MessageProcessorTypeDeclarationXmlLoader messageProcessorTypeDeclarationXmlLoader;
  private final ExtensionOperationTypeDeclarationXmlLoader extensionOperationTypeDeclarationXmlLoader;
  private final GlobalBindingMetadataTypes globalBindingMetadataTypes;

  private Metadata apiKitMetadata = null;
  private org.mule.module.soapkit.metadata.api.Metadata soapKitMetadata = null;

  public DataSenseProviderResolver(DataSenseProvider dataSenseProvider, ApplicationModelResolver applicationModelResolver,
                                   AstNotification astNotification) {
    this.dataSenseProvider = dataSenseProvider;
    this.applicationModelResolver = applicationModelResolver;
    this.astNotification = astNotification;
    notifier = new Notifier() {

      @Override
      public void reportWarning(ComponentLocation componentLocation, I18nMessage i18nMessage) {
        astNotification.reportWarning(new AstNodeLocation(componentLocation), i18nMessage);
      }

      @Override
      public void reportWarning(I18nMessage i18nMessage) {
        astNotification.reportWarning(i18nMessage);
      }

      @Override
      public void reportError(ComponentLocation componentLocation, I18nMessage i18nMessage) {
        astNotification.reportError(new AstNodeLocation(componentLocation), i18nMessage);
      }

      @Override
      public void reportError(I18nMessage i18nMessage) {
        astNotification.reportError(i18nMessage);
      }

      @Override
      public void reportFatalError(ComponentLocation componentLocation, I18nMessage i18nMessage) {
        astNotification.reportFatalError(new AstNodeLocation(componentLocation), i18nMessage);
      }

      @Override
      public void reportFatalError(I18nMessage i18nMessage) {
        astNotification.reportFatalError(i18nMessage);
      }
    };

    final ApplicationModel applicationModel = applicationModelResolver.getApplicationModel();
    final List<String> typesDataList = applicationModel.findTypesDataList();

    typesCatalog = loadTypesCatalog(applicationModel, typesDataList);
    componentModelEnrichments = loadComponentModelEnrichments(typesDataList);
    messageProcessorTypeDeclarationXmlLoader = new MessageProcessorTypeDeclarationXmlLoader();
    extensionOperationTypeDeclarationXmlLoader = new ExtensionOperationTypeDeclarationXmlLoader();

    try {
      buildApiKitMetadata(dataSenseProvider, astNotification, applicationModel);
    } catch (Exception e) {
      astNotification.reportError(null, NotificationMessages.MSG_RAML_RESOLUTION_INITIALIZATION(e.getMessage()));
    }

    try {
      buildSoapKitMetadata(astNotification, applicationModel);
    } catch (Exception e) {
      astNotification.reportError(null, NotificationMessages.MSG_SOAPKIT_RESOLUTION_INITIALIZATION(e.getMessage()));
    }

    globalBindingMetadataTypes = new GlobalBindingMetadataTypes(getExpressionLanguageMetadataService());
  }

  private ComponentModelEnrichments loadComponentModelEnrichments(List<String> typesDataList) {
    ComponentModelEnrichments componentModelEnrichments = new ComponentModelEnrichments();
    ComponentModelEnrichmentXmlLoader componentModelEnrichmentXmlLoader = new ComponentModelEnrichmentXmlLoader();
    typesDataList.forEach(typesData -> {
      loadComponentModelEnrichments(componentModelEnrichments, typesData, componentModelEnrichmentXmlLoader);
    });
    return componentModelEnrichments;
  }

  private void loadComponentModelEnrichments(ComponentModelEnrichments componentModelEnrichments, String typesData,
                                             ComponentModelEnrichmentXmlLoader componentModelEnrichmentXmlLoader) {
    final Element element;
    try {
      element = XmlUtils.parseRootElement(new InputSource(new StringReader(typesData)), true, false);

      componentModelEnrichmentXmlLoader.load(componentModelEnrichments, element, new ComponentModelEnrichmentLoaderContext());
    } catch (Exception e) {
      throw Throwables.propagate(e);
    }
  }

  private TypesCatalog loadTypesCatalog(ApplicationModel applicationModel, List<String> typesDataList) {
    try {
      TypesCatalogBuilderContext typesCatalogBuilderContext = new TypesCatalogBuilderContext(notifier,
                                                                                             dataSenseProvider
                                                                                                 .getExpressionLanguageMetadataService());
      return new TypesCatalogXmlLoader().loadData(typesDataList, typesCatalogBuilderContext,
                                                  applicationModel.findResource("").orElse(null));
    } catch (Exception e) {
      throw Throwables.propagate(e);
    }
  }

  private void buildApiKitMetadata(final DataSenseProvider dataSenseProvider, final AstNotification astNotification,
                                   final ApplicationModel applicationModel) {
    final MetadataBuilder apiKitMetadataBuilder = dataSenseProvider.getApikitMetadataBuilder();
    apiKitMetadataBuilder.withApplicationModel(applicationModel.getMuleApplicationModel());
    apiKitMetadataBuilder.withResourceLoader(resource -> {
      if (!resource.startsWith("resource::")) {
        try {
          new URI(resource);
        } catch (URISyntaxException e) {
          try {
            resource = new URI(null, null, resource, null).getRawPath();
          } catch (URISyntaxException e1) {
          }
        }
        try {
          resource = new URI("api/").resolve(resource).getRawPath();
        } catch (URISyntaxException e) {
        }
      }
      return applicationModel.findResource(resource).orElse(null);
    });
    apiKitMetadataBuilder.withNotifier(new org.mule.runtime.apikit.metadata.api.Notifier() {

      @Override
      public void error(String s) {
        astNotification.reportError(NotificationMessages.MSG_RAML_RESOLUTION_NOTIFICATION("error", s));
      }

      @Override
      public void warn(String s) {
        astNotification.reportWarning(NotificationMessages.MSG_RAML_RESOLUTION_NOTIFICATION("warning", s));

      }

      @Override
      public void info(String s) {
        astNotification.reportInfo(NotificationMessages.MSG_RAML_RESOLUTION_NOTIFICATION("info", s));

      }

      @Override
      public void debug(String s) {
        astNotification.reportDebug(NotificationMessages.MSG_RAML_RESOLUTION_NOTIFICATION("trace", s));
      }
    });
    apiKitMetadata = apiKitMetadataBuilder.build();
  }

  private void buildSoapKitMetadata(final AstNotification astNotification, final ApplicationModel applicationModel) {
    final org.mule.module.soapkit.metadata.api.Metadata.Builder soapKitMetadataBuilder =
        new org.mule.module.soapkit.metadata.api.Metadata.Builder();
    soapKitMetadataBuilder.withApplicationModel(applicationModel.getMuleApplicationModel());
    soapKitMetadataBuilder.withResourceLoader(resource -> {
      try {
        resource = new URI("api/").resolve(resource).toString();
      } catch (URISyntaxException e) {
      }
      return applicationModel.findResource(resource).map(uri -> {
        try {
          return uri.toURL();
        } catch (MalformedURLException e) {
          return null;
        }
      }).orElse(null);
    });
    soapKitMetadataBuilder.withNotifier(new org.mule.module.soapkit.metadata.api.Notifier() {

      @Override
      public void error(String s) {
        astNotification.reportError(NotificationMessages.MSG_SOAPKIT_RESOLUTION_NOTIFICATION("error", s));
      }

      @Override
      public void warn(String s) {
        astNotification.reportWarning(NotificationMessages.MSG_SOAPKIT_RESOLUTION_NOTIFICATION("warning", s));

      }

      @Override
      public void info(String s) {
        astNotification.reportInfo(NotificationMessages.MSG_SOAPKIT_RESOLUTION_NOTIFICATION("info", s));

      }

      @Override
      public void debug(String s) {
        astNotification.reportDebug(NotificationMessages.MSG_SOAPKIT_RESOLUTION_NOTIFICATION("trace", s));
      }
    });
    soapKitMetadata = soapKitMetadataBuilder.build();
  }

  public DataSenseProvider getDataSenseProvider() {
    return dataSenseProvider;
  }

  public ApplicationModelResolver getApplicationModelResolver() {
    return applicationModelResolver;
  }

  public ExpressionLanguageMetadataService getExpressionLanguageMetadataService() {
    return dataSenseProvider.getExpressionLanguageMetadataService();
  }

  public Optional<TypesCatalog> getTypesCatalog() {
    return Optional.ofNullable(typesCatalog);
  }

  public Optional<ComponentModelEnrichments> getComponentModelEnrichments() {
    return Optional.ofNullable(componentModelEnrichments);
  }

  /**
   * @param componentModel
   * @return
   */
  public Optional<SourceModel> resolveSourceModel(ComponentAst componentModel) {
    return componentModel.getModel(SourceModel.class);
  }

  /**
   * @param componentModel
   * @return
   */
  public Optional<OperationModel> resolveOperationModel(ComponentAst componentModel) {
    return componentModel.getModel(OperationModel.class);
  }

  private <T> Optional<T> findTypeDeclaration(ComponentAst componentModel,
                                              TypeDeclarationXmlLoader<T, Element> typeDeclarationLoader) {
    return getComponentModelEnrichments()
        .flatMap(componentModelEnrichments1 -> componentModelEnrichments1.enrich(componentModel).map(o -> (Element) o)
            .map(element -> typeDeclarationLoader.load(element,
                                                       new TypeDeclarationLoaderContext(componentModel, typesCatalog, notifier)))
            .filter(Optional::isPresent)
            .map(Optional::get)
            .findFirst());
  }

  public Optional<MessageProcessorTypeDeclaration> findMessageProcessorTypeDeclaration(ComponentAst componentModel) {
    return findTypeDeclaration(componentModel, messageProcessorTypeDeclarationXmlLoader);
  }

  public Optional<ExtensionOperationTypeDeclaration> findExtensionOperationTypeDeclaration(ComponentAst componentModel) {
    return findTypeDeclaration(componentModel, extensionOperationTypeDeclarationXmlLoader);
  }

  public AstNotification getAstNotification() {
    return astNotification;
  }

  public Optional<Metadata> getApiKitMetadata() {
    return Optional.ofNullable(apiKitMetadata);
  }

  public Optional<org.mule.module.soapkit.metadata.api.Metadata> getSoapKitMetadata() {
    return Optional.ofNullable(soapKitMetadata);
  }

  public Function<String, Optional<URI>> getResourceResolver() {
    return resource -> getApplicationModelResolver().getApplicationModel().findResource(resource);
  }

  public GlobalBindingMetadataTypes getGlobalBindingMetadataTypes() {
    return globalBindingMetadataTypes;
  }
}

==> mule-datasense-4.9.9-20250724.133837-1-sources.jar: org/mule/datasense/impl/DataSenseProviderResolver.java <==
/*
 * Copyright (c) MuleSoft, Inc.  All rights reserved.  http://www.mulesoft.com
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.datasense.impl;

import org.mule.datasense.api.metadataprovider.ApplicationModel;
import org.mule.datasense.api.metadataprovider.DataSenseProvider;
import org.mule.datasense.catalog.builder.TypesCatalogBuilderContext;
import org.mule.datasense.catalog.loader.xml.TypesCatalogXmlLoader;
import org.mule.datasense.catalog.model.TypesCatalog;
import org.mule.datasense.common.loader.xml.XmlUtils;
import org.mule.datasense.common.util.Notifier;
import org.mule.datasense.declarations.loader.TypeDeclarationLoaderContext;
import org.mule.datasense.declarations.loader.xml.ExtensionOperationTypeDeclarationXmlLoader;
import org.mule.datasense.declarations.loader.xml.MessageProcessorTypeDeclarationXmlLoader;
import org.mule.datasense.declarations.loader.xml.TypeDeclarationXmlLoader;
import org.mule.datasense.declarations.model.ExtensionOperationTypeDeclaration;
import org.mule.datasense.declarations.model.MessageProcessorTypeDeclaration;
import org.mule.datasense.enrichment.loader.ComponentModelEnrichmentLoaderContext;
import org.mule.datasense.enrichment.loader.xml.ComponentModelEnrichmentXmlLoader;
import org.mule.datasense.enrichment.model.ComponentModelEnrichments;
import org.mule.datasense.impl.model.ast.AstNodeLocation;
import org.mule.datasense.impl.model.ast.AstNotification;
import org.mule.datasense.impl.model.ast.MessageProcessorNode;
import org.mule.datasense.impl.model.reporting.NotificationMessages;
import org.mule.datasense.impl.phases.typing.resolver.GlobalBindingMetadataTypes;
import org.mule.runtime.api.component.ComponentIdentifier;
import org.mule.runtime.api.component.location.ComponentLocation;
import org.mule.runtime.api.i18n.I18nMessage;
import org.mule.runtime.api.meta.model.operation.OperationModel;
import org.mule.runtime.api.meta.model.source.SourceModel;
import org.mule.runtime.api.metadata.ExpressionLanguageMetadataService;
import org.mule.runtime.apikit.metadata.api.Metadata;
import org.mule.runtime.apikit.metadata.api.MetadataBuilder;
import org.mule.runtime.ast.api.ComponentAst;

import java.io.StringReader;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

import com.google.common.base.Throwables;

import org.w3c.dom.Element;
import org.xml.sax.InputSource;

/**
 *
 */
public class DataSenseProviderResolver {

  private static final List<ComponentIdentifier> componentIdentifierBlockList = Collections.emptyList();

  public static boolean isDynamicDataSenseSupportedFor(MessageProcessorNode messageProcessorNode) {
    return !componentIdentifierBlockList.contains(messageProcessorNode.getComponentIdentifier());
  }

  private final DataSenseProvider dataSenseProvider;
  private final ApplicationModelResolver applicationModelResolver;
  private final AstNotification astNotification;
  private final Notifier notifier;

  private final TypesCatalog typesCatalog;
  private final ComponentModelEnrichments componentModelEnrichments;
  private final MessageProcessorTypeDeclarationXmlLoader messageProcessorTypeDeclarationXmlLoader;
  private final ExtensionOperationTypeDeclarationXmlLoader extensionOperationTypeDeclarationXmlLoader;
  private final GlobalBindingMetadataTypes globalBindingMetadataTypes;

  private Metadata apiKitMetadata = null;
  private org.mule.module.soapkit.metadata.api.Metadata soapKitMetadata = null;

  public DataSenseProviderResolver(DataSenseProvider dataSenseProvider, ApplicationModelResolver applicationModelResolver,
                                   AstNotification astNotification) {
    this.dataSenseProvider = dataSenseProvider;
    this.applicationModelResolver = applicationModelResolver;
    this.astNotification = astNotification;
    notifier = new Notifier() {

      @Override
      public void reportWarning(ComponentLocation componentLocation, I18nMessage i18nMessage) {
        astNotification.reportWarning(new AstNodeLocation(componentLocation), i18nMessage);
      }

      @Override
      public void reportWarning(I18nMessage i18nMessage) {
        astNotification.reportWarning(i18nMessage);
      }

      @Override
      public void reportError(ComponentLocation componentLocation, I18nMessage i18nMessage) {
        astNotification.reportError(new AstNodeLocation(componentLocation), i18nMessage);
      }

      @Override
      public void reportError(I18nMessage i18nMessage) {
        astNotification.reportError(i18nMessage);
      }

      @Override
      public void reportFatalError(ComponentLocation componentLocation, I18nMessage i18nMessage) {
        astNotification.reportFatalError(new AstNodeLocation(componentLocation), i18nMessage);
      }

      @Override
      public void reportFatalError(I18nMessage i18nMessage) {
        astNotification.reportFatalError(i18nMessage);
      }
    };

    final ApplicationModel applicationModel = applicationModelResolver.getApplicationModel();
    final List<String> typesDataList = applicationModel.findTypesDataList();

    typesCatalog = loadTypesCatalog(applicationModel, typesDataList);
    componentModelEnrichments = loadComponentModelEnrichments(typesDataList);
    messageProcessorTypeDeclarationXmlLoader = new MessageProcessorTypeDeclarationXmlLoader();
    extensionOperationTypeDeclarationXmlLoader = new ExtensionOperationTypeDeclarationXmlLoader();

    try {
      buildApiKitMetadata(dataSenseProvider, astNotification, applicationModel);
    } catch (Exception e) {
      astNotification.reportError(null, NotificationMessages.MSG_RAML_RESOLUTION_INITIALIZATION(e.getMessage()));
    }

    try {
      buildSoapKitMetadata(astNotification, applicationModel);
    } catch (Exception e) {
      astNotification.reportError(null, NotificationMessages.MSG_SOAPKIT_RESOLUTION_INITIALIZATION(e.getMessage()));
    }

    globalBindingMetadataTypes = new GlobalBindingMetadataTypes(getExpressionLanguageMetadataService());
  }

  private ComponentModelEnrichments loadComponentModelEnrichments(List<String> typesDataList) {
    ComponentModelEnrichments componentModelEnrichments = new ComponentModelEnrichments();
    ComponentModelEnrichmentXmlLoader componentModelEnrichmentXmlLoader = new ComponentModelEnrichmentXmlLoader();
    typesDataList.forEach(typesData -> {
      loadComponentModelEnrichments(componentModelEnrichments, typesData, componentModelEnrichmentXmlLoader);
    });
    return componentModelEnrichments;
  }

  private void loadComponentModelEnrichments(ComponentModelEnrichments componentModelEnrichments, String typesData,
                                             ComponentModelEnrichmentXmlLoader componentModelEnrichmentXmlLoader) {
    final Element element;
    try {
      element = XmlUtils.parseRootElement(new InputSource(new StringReader(typesData)), true, false);

      componentModelEnrichmentXmlLoader.load(componentModelEnrichments, element, new ComponentModelEnrichmentLoaderContext());
    } catch (Exception e) {
      throw Throwables.propagate(e);
    }
  }

  private TypesCatalog loadTypesCatalog(ApplicationModel applicationModel, List<String> typesDataList) {
    try {
      TypesCatalogBuilderContext typesCatalogBuilderContext = new TypesCatalogBuilderContext(notifier,
                                                                                             dataSenseProvider
                                                                                                 .getExpressionLanguageMetadataService());
      return new TypesCatalogXmlLoader().loadData(typesDataList, typesCatalogBuilderContext,
                                                  applicationModel.findResource("").orElse(null));
    } catch (Exception e) {
      throw Throwables.propagate(e);
    }
  }

  private void buildApiKitMetadata(final DataSenseProvider dataSenseProvider, final AstNotification astNotification,
                                   final ApplicationModel applicationModel) {
    final MetadataBuilder apiKitMetadataBuilder = dataSenseProvider.getApikitMetadataBuilder();
    apiKitMetadataBuilder.withApplicationModel(applicationModel.getMuleApplicationModel());
    apiKitMetadataBuilder.withResourceLoader(resource -> {
      String originalResource = resource;
      if (!resource.startsWith("resource::")) {
        try {
          new URI(resource);
        } catch (URISyntaxException e) {
          try {
            resource = new URI(null, null, resource, null).getRawPath();
          } catch (URISyntaxException e1) {
          }
        }
        try {
          resource = new URI("api/").resolve(resource).getRawPath();
        } catch (URISyntaxException e) {
        }
      }
      return applicationModel.findResource(resource)
          .orElseGet(() -> applicationModel.findResource(originalResource).orElse(null));
    });
    apiKitMetadataBuilder.withNotifier(new org.mule.runtime.apikit.metadata.api.Notifier() {

      @Override
      public void error(String s) {
        astNotification.reportError(NotificationMessages.MSG_RAML_RESOLUTION_NOTIFICATION("error", s));
      }

      @Override
      public void warn(String s) {
        astNotification.reportWarning(NotificationMessages.MSG_RAML_RESOLUTION_NOTIFICATION("warning", s));

      }

      @Override
      public void info(String s) {
        astNotification.reportInfo(NotificationMessages.MSG_RAML_RESOLUTION_NOTIFICATION("info", s));

      }

      @Override
      public void debug(String s) {
        astNotification.reportDebug(NotificationMessages.MSG_RAML_RESOLUTION_NOTIFICATION("trace", s));
      }
    });
    apiKitMetadata = apiKitMetadataBuilder.build();
  }

  private void buildSoapKitMetadata(final AstNotification astNotification, final ApplicationModel applicationModel) {
    final org.mule.module.soapkit.metadata.api.Metadata.Builder soapKitMetadataBuilder =
        new org.mule.module.soapkit.metadata.api.Metadata.Builder();
    soapKitMetadataBuilder.withApplicationModel(applicationModel.getMuleApplicationModel());
    soapKitMetadataBuilder.withResourceLoader(resource -> {
      String originalResource = resource;
      try {
        resource = new URI("api/").resolve(resource).toString();
      } catch (URISyntaxException e) {
      }
      return applicationModel.findResource(resource).map(uri -> {
        try {
          return uri.toURL();
        } catch (MalformedURLException e) {
          return null;
        }
      }).orElseGet(() -> {
        return applicationModel.findResource(originalResource).map(uri -> {
          try {
            return uri.toURL();
          } catch (MalformedURLException e) {
            return null;
          }
        }).orElse(null);
      });
    });
    soapKitMetadataBuilder.withNotifier(new org.mule.module.soapkit.metadata.api.Notifier() {

      @Override
      public void error(String s) {
        astNotification.reportError(NotificationMessages.MSG_SOAPKIT_RESOLUTION_NOTIFICATION("error", s));
      }

      @Override
      public void warn(String s) {
        astNotification.reportWarning(NotificationMessages.MSG_SOAPKIT_RESOLUTION_NOTIFICATION("warning", s));

      }

      @Override
      public void info(String s) {
        astNotification.reportInfo(NotificationMessages.MSG_SOAPKIT_RESOLUTION_NOTIFICATION("info", s));

      }

      @Override
      public void debug(String s) {
        astNotification.reportDebug(NotificationMessages.MSG_SOAPKIT_RESOLUTION_NOTIFICATION("trace", s));
      }
    });
    soapKitMetadata = soapKitMetadataBuilder.build();
  }

  public DataSenseProvider getDataSenseProvider() {
    return dataSenseProvider;
  }

  public ApplicationModelResolver getApplicationModelResolver() {
    return applicationModelResolver;
  }

  public ExpressionLanguageMetadataService getExpressionLanguageMetadataService() {
    return dataSenseProvider.getExpressionLanguageMetadataService();
  }

  public Optional<TypesCatalog> getTypesCatalog() {
    return Optional.ofNullable(typesCatalog);
  }

  public Optional<ComponentModelEnrichments> getComponentModelEnrichments() {
    return Optional.ofNullable(componentModelEnrichments);
  }

  /**
   * @param componentModel
   * @return
   */
  public Optional<SourceModel> resolveSourceModel(ComponentAst componentModel) {
    return componentModel.getModel(SourceModel.class);
  }

  /**
   * @param componentModel
   * @return
   */
  public Optional<OperationModel> resolveOperationModel(ComponentAst componentModel) {
    return componentModel.getModel(OperationModel.class);
  }

  private <T> Optional<T> findTypeDeclaration(ComponentAst componentModel,
                                              TypeDeclarationXmlLoader<T, Element> typeDeclarationLoader) {
    return getComponentModelEnrichments()
        .flatMap(componentModelEnrichments1 -> componentModelEnrichments1.enrich(componentModel).map(o -> (Element) o)
            .map(element -> typeDeclarationLoader.load(element,
                                                       new TypeDeclarationLoaderContext(componentModel, typesCatalog, notifier)))
            .filter(Optional::isPresent)
            .map(Optional::get)
            .findFirst());
  }

  public Optional<MessageProcessorTypeDeclaration> findMessageProcessorTypeDeclaration(ComponentAst componentModel) {
    return findTypeDeclaration(componentModel, messageProcessorTypeDeclarationXmlLoader);
  }

  public Optional<ExtensionOperationTypeDeclaration> findExtensionOperationTypeDeclaration(ComponentAst componentModel) {
    return findTypeDeclaration(componentModel, extensionOperationTypeDeclarationXmlLoader);
  }

  public AstNotification getAstNotification() {
    return astNotification;
  }

  public Optional<Metadata> getApiKitMetadata() {
    return Optional.ofNullable(apiKitMetadata);
  }

  public Optional<org.mule.module.soapkit.metadata.api.Metadata> getSoapKitMetadata() {
    return Optional.ofNullable(soapKitMetadata);
  }

  public Function<String, Optional<URI>> getResourceResolver() {
    return resource -> getApplicationModelResolver().getApplicationModel().findResource(resource);
  }

  public GlobalBindingMetadataTypes getGlobalBindingMetadataTypes() {
    return globalBindingMetadataTypes;
  }
}
